<?php

namespace Drp\UserManagement\Http\Controllers\Admin;

use Drp\UserManagement\Contracts\ManagedUser;
use Drp\UserManagement\Http\Requests\CreateUser;
use Drp\UserManagement\UserCreator;
use Illuminate\Foundation\Auth\Access\AuthorizesRequests;
use Illuminate\Foundation\Bus\DispatchesJobs;
use Illuminate\Foundation\Validation\ValidatesRequests;
use Illuminate\Http\RedirectResponse;
use Illuminate\Routing\Controller;
use Illuminate\View\View;

class UsersController extends Controller
{
    use AuthorizesRequests, DispatchesJobs, ValidatesRequests;

    /**
     * @var ManagedUser
     */
    private $user;

    public function __construct(ManagedUser $user)
    {
        $this->user = $user;
    }

    public function index(): View
    {
        $users = $this->user->get();
        $awaitingApproval = $this->user->allAwaitingApproval();
        return view('userManagement::users.index', compact('users', 'awaitingApproval'));
    }

    public function create(): View
    {
        return view('userManagement::users.create');
    }

    public function store(CreateUser $request, UserCreator $userCreator): RedirectResponse
    {
        $userCreator->createWithLoginCode($request->input());

        flash()->success(trans('userManagement::users.created'));

        return redirect()
            ->route('admin.users.index');
    }

    // phpcs:disable Generic.CodeAnalysis.UnusedFunctionParameter.Found

    public function edit(ManagedUser $user): View
    {
        return view('userManagement::users.edit', compact('user'));
    }

    // phpcs:enable Generic.CodeAnalysis.UnusedFunctionParameter.Found

    public function update(CreateUser $request, ManagedUser $user): RedirectResponse
    {
        $user->update($request->input());

        flash()->success(trans('userManagement::users.updated'));

        return redirect()->route('admin.users.index');
    }

    public function destroy(ManagedUser $user): RedirectResponse
    {
        $this->authorize('delete', $user);

        $user->delete();

        flash()->success(trans('userManagement::users.deleted'));

        return redirect()->route('admin.users.index');
    }
}
